require 'yaml'
require 'mu/result'

module Streetlib
  module Vault

    class Loader
      include Mu
      attr_accessor :path, :content
    
      def initialize(path:)
        self.path = path
        self.content = nil
      end
    
      def self.load(path:)
        instance = new(path: path)
        result = instance.load
        return result if result.error?
    
        Result.success(instance)
      end
    
      def load
        context = { path: path }
        return Result.error(context).code!(:no_such_file) unless File.exists?(path)
    
        begin
          yaml_content = YAML.load_file(path)
        rescue StandardError => error
          return Result.
            error(context.merge({ error: error })).
            code!(:cant_load_file)
        end
    
        self.content = Content.new(yaml_content)
        Result.success
      end
    end

  end
end